#include <qapplication.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qwidget.h>
#include <qtimer.h>

#include "settings.h"

#include  <stdlib.h>
#include  <stdio.h>
#include  <math.h>

#define MAXPOINTS 100


extern Config config[];

class Surface
{
  float y[MAXPOINTS],y_p[MAXPOINTS];
  
 public:
  Surface(void);
  float get(unsigned i) { return y[i]; }
  void  force(float y);
  void  clear();
};

class GolfWindow : public QWidget
{
 Q_OBJECT
   
 public:
  GolfWindow();
  ~GolfWindow();
	  
 protected:
  virtual void mousePressEvent( QMouseEvent *);
  virtual void paintEvent( QPaintEvent *);
  virtual void resizeEvent( QResizeEvent *);

 private slots:
   void slotAbout();
   void slotAboutQt();
   void slotClear(); 
   void slotUpdate();
   void slotSettings();
   void slotReset();
   void slotFlush();
   
 private:
  QPixmap     _buffer;
  QTimer     *_timer;
  QMenuBar   *_menubar;
  QPopupMenu *_filemenu;
  QPopupMenu *_helpmenu;
  QPopupMenu *_popupmenu;

  Settings   *_settings;

  float   t;                    /*current time */
  Surface surf;                 /*Surface of wave (1D here)*/
};

#include "qgolf.moc"

Surface::Surface(void) 
{
  memset(y,0,sizeof(y)); 
  memset(y_p,0,sizeof(y_p));
}

void
Surface::clear()
{ 
  memset(y,0,sizeof(y)); 
  memset(y_p,0,sizeof(y_p));
}

void
Surface::force(float ym) 
{
  float tmp=0.0,v=0.0,yy=0.0,a=0.0;
  char s[10];
 
  /* upload settings 
   */
  float dt=0.0, dx=0.0, f=0.0, mu=0.0;
  int maxpoints=0;

  dt            = atof( config[CONFIG_DT].val);
  maxpoints     = atoi( config[CONFIG_MAXPOINTS].val);
  dx            = atof( config[CONFIG_DX].val);
  f             = atof( config[CONFIG_F].val);
  mu            = atof( config[CONFIG_MU].val);

  if (fabs(mu)<ZERO_TRESH || maxpoints<=2 || maxpoints>=MAXPOINTS || fabs(dx)<ZERO_TRESH) 
  {
    return;
  }
    
  /* Bereken puls: kracht nodig op uiteinde koord van rust 
     naar een positie y (geven) te brengen..
   */
  
  a =  (ym-y[0])/(dt*dt);
  y_p[0] =  y[0];
  y[0] = ym;
  
  for (int i=1; i<maxpoints-1; ++i)
  {
    
    yy = y[i];  
    v  = (yy-y_p[i])/dt;
    a  = (f/mu)*( (y[i+1]-yy) - (yy-y_p[i-1]) )/dx;  //carefull! y[i-1] has been overwritten. use old value!  
    
    y[i] = yy + dt*( v+  0.5*a*dt); 
    y_p[i]=yy;
#ifdef _DEBUG
    printf("i:%d a:%2.03f v:%2.03f x:%2.03f => %2.03f",i,a,v,yy,y[i]);
    printf("\n");
    gets(s);
#endif    
  }

  switch (config[ CONFIG_END].val[0])
  {
    /* diff. closed and open cord termination
     */
    case 'c':
    case 'C':
      y[maxpoints-1] = 0.0;
      break;
      
    default:
      y[maxpoints-1] = y[maxpoints-2];
      break;
  }
}


GolfWindow::GolfWindow() : t(0),_settings(0)
{
  setBackgroundMode( NoBackground );

  _filemenu = new QPopupMenu;
  _filemenu->insertItem( "&Settings", this,  SLOT(slotSettings()) );
  _filemenu->insertItem( "&Quit", qApp, SLOT( quit() ) );

  _helpmenu = new QPopupMenu;
  _helpmenu->insertItem("&About QtScribble", this, SLOT(slotAbout() ) );
  _helpmenu->insertItem("About &Qt", this, SLOT(slotAboutQt() ) );

  _menubar = new QMenuBar(this);
  _menubar->insertItem( "&File", _filemenu );
  _menubar->insertSeparator();
  _menubar->insertItem( "&Help", _helpmenu );

  _popupmenu = new QPopupMenu;
  _popupmenu->insertItem( "&Clear", this, SLOT(slotClear()) );
  _popupmenu->insertItem( "&Flush", this, SLOT(slotFlush()) );
  _popupmenu->insertItem( "&Settings", this,  SLOT(slotSettings()) );
 
   /* start timer....
    */
   _timer = new QTimer( this );
   connect( _timer, SIGNAL(timeout()), this , SLOT(slotUpdate()) );

   int dtsimul = atoi( config[CONFIG_DTSIMUL].val );
   _timer->start( dtsimul, FALSE );   
}

GolfWindow::~GolfWindow()
{
  delete _popupmenu;
}

void GolfWindow::slotClear()
{
  _buffer.fill(white);
  t = 0;
  bitBlt( this, 0,0, &_buffer );
}

void GolfWindow::slotFlush()
{
  surf.clear();
}

void GolfWindow::slotReset()
{
 slotClear();
 t = 0.0;

 float msec = atoi( config[ CONFIG_DTSIMUL ].val );
 if (msec>5)  /* let's be realistic .. */
   _timer->changeInterval( msec );
}

void GolfWindow::slotSettings()
{
   /* make config-window
    */
  if (_settings==NULL)
     _settings = new Settings(this,"settings",TRUE);
  
   _settings->exec();
}

void GolfWindow::mousePressEvent( QMouseEvent *ev)
{
  if (ev->button() == RightButton )
   _popupmenu->exec( QCursor::pos() );
}


void GolfWindow::slotUpdate()
{
  QPainter bufpaint;
  float    y=0,oldy=0,tt=0.0;
  unsigned vx=50,vy=height()>>1,vdx;

  /* UPLOAD Configuration settings
   */
  float dt=0.0;
  int   maxpoints=10;
  dt = atof( config[ CONFIG_DT ].val );
  maxpoints = atoi( config[CONFIG_MAXPOINTS].val );
  
  if (maxpoints<=1 || maxpoints>MAXPOINTS)
  {
    fprintf(stderr,"Pls enter normal config settings.");
    return;
  }
  
 
  /*  decoration ...
   */ 
  if (width()<50 || height()<50)
    return;                             /*canvas is too small*/

  vdx = (width()-(vx<<1))/maxpoints;
  if (vdx<=0) vdx=1;

  
  /* Force a function and calculate rope behaviour.
   */

  switch ( config[ CONFIG_FUNC ].val[0] )
  {
    case 's':
    case 'S':
      surf.force( 20*sin( t ) );
      break;
    case 'c':
      surf.force( 20*cos(1.123*t)*sin( t ) );
      break;
     case 'p':
    case 'P':
      if (t>=1.0 && t<1.5)
        surf.force( 20 );
      else 
        surf.force( 0.0  );
      break;

    default:
      surf.force( 0.0 );
      break;
  }

  
  /* paint it !
   */
  _buffer.fill(white);
  bufpaint.begin( &_buffer );
  bufpaint.setPen( darkBlue );

  for (int i=0; i<maxpoints-1; ++i) {
    y = surf.get(i);
    
    if (i==0) oldy=y; 
    bufpaint.drawLine( QPoint(vdx*i+vx, oldy+vy), QPoint(vdx*(i+1)+vx, y+vy) );
    oldy = y;
  }
  t += dt;
  
  /*  show it !
   */
  bufpaint.end();
  bitBlt(this,0,0,&_buffer);
}


void GolfWindow::paintEvent( QPaintEvent *ev)
{
  bitBlt(this, 0,0, &_buffer );
}

void GolfWindow::resizeEvent( QResizeEvent *ev)
{
  _buffer.resize( ev->size() );
  slotClear();
}



void GolfWindow::slotAbout()
{
  QMessageBox::information( this, "About QGolf", "This is wave simulator\nCopyright 2001 by Hans Opstaele" );
}

void GolfWindow::slotAboutQt()
{
  QMessageBox::aboutQt( this, "About Qt" );
}


int main( int argc, char *argv[])
{
  QApplication myapp(argc,argv);

  GolfWindow *mywidget = new GolfWindow();
  mywidget->setGeometry( 50,50,600,200);
  myapp.setMainWidget( mywidget );
  mywidget->show();
  return myapp.exec();
}


